@@ -9,7 +9,7 @@ Test basic changelog functionality: Ansible collections
 
 import os
 
-from typing import List
+from typing import List, Optional
 
 import mock
 
@@ -969,7 +969,9 @@ foo
 ''')
 
 
-def fake_ansible_doc_collection(paths: PathsConfig, plugin_type: str,
+def fake_ansible_doc_collection(paths: PathsConfig,
+                                playbook_dir: Optional[str],
+                                plugin_type: str,
                                 plugin_names: List[str]) -> dict:
     """
     Fake ansible-doc mock for collection.
